<?php
namespace tests2;

require_once 'PHPUnit/Autoload.php';
require_once 'functional.php';

use kingfisher\functional as fp;

class Tests extends \PHPUnit_Framework_TestCase {
  function test1() {
    $x = fp\curry(ns('f1'));
    foreach (range(1, 3) as $n) {
      $x = $x($n);
    }
    $this->assertEquals($x, array(1, 2, 3));
  }

  function test2() {
    $x = fp\curry(ns('f1'), 7);
    foreach (range(8, 9) as $n)
      $x = $x($n);
    $this->assertEquals($x, array(7, 8, 9));
  }

  function test3() {
    $this->setExpectedException(
      'Exception',
      "argument 0 (counting from 0) must be an array"
    );
    $x = fp\curry(ns('f2'));
    $x = $x(1);
  }

  function test_array_and_hint1() {
    $x = fp\safe_spatter(ns('takes_array_or_NULL'), 1);
    #var_dump($x);
    $x = $x(NULL);
  }

  function test_array_and_hint2() {
    $this->setExpectedException(
      'Exception',
      "argument 0 (counting from 0) must be NULL or an array"
    );
    $x = fp\safe_spatter(ns('takes_array_or_NULL'), 1);
    $x = $x(1);
  }

  function test4() {
    $this->setExpectedException(
      'Exception',
      "argument 1 (counting from 0) must be an instance of DateTime"
    );
    $x = fp\curry(ns('f2'));
    $x = $x(range(3, 4));
    $x = $x(44);
  }

  function test5() {
    $x = fp\curry(ns('f2'));
    $x = $x(range(3, 4));
    $x = $x(new \DateTime('today'));
    $x('last');
  }

  function test_object_and_hint1() {
    $x = fp\safe_spatter(ns('takes_DateTime_or_NULL'), 1);
    $x = $x(NULL);
  }

  function test_object_and_hint2() {
    $this->setExpectedException(
      'Exception',
      "argument 0 (counting from 0) must be " .
        "NULL or an instance of DateTime"
    );
    $x = fp\safe_spatter(ns('takes_DateTime_or_NULL'), 1);
    $x = $x(1);
  }

  function test6() {
    $this->setExpectedException(
      'Exception',
      'Too many arguments: 4 provided, maximum of 2 allowed here'
    );
    fp\curry(ns('f1'), 1, 2, 3, 4);
  }

  function test7() {
    $orig = fp\curry(ns('f1'));
    $f = $orig;
    $f(1, 2, 3);
    $f = $orig;
    $f(11, 12, 13);
  }

  function test8() {
    $x = fp\curry(array(ns('c'), 'f1'));
    $x = $x(1);
    $x(2, 3);
    $this->assertEquals($x(2, 3), range(1, 3));
  }

  function test_saturation1() {
    $x = fp\curry(ns('f1'), 1, 2, 3);
    $this->assertEquals($x(), range(1, 3));
  }

  function test_saturation2() {
    $x = fp\curry('php_uname');
    $this->assertInternalType('string', $x());
  }

  function test_saturation3() {
    $x = fp\curry(ns('php_uname_wrapper'));
    $this->assertInternalType('string', $x());
  }

  function test_saturation4() {
    $x = fp\safe_spatter(ns('php_uname_wrapper'), array('a'));
    $this->assertInternalType('string', $x());
  }

  function test10() {
    $this->setExpectedException(
      'Exception',
      'Too many arguments: 4 provided, maximum of 2 allowed here'
    );
    fp\curry(ns('f1'), 1, 2, 3, 4);
  }

  function test11() {
    $this->setExpectedException(
      'Exception',
      "function has been specified with too few " .
        "arguments: " .
        "number specified: 2; number required: between 3 and 4"
    );
    fp\safe_spatter(ns('f1'), 1, array(1));
  }

  function test12() {
    $x = fp\safe_spatter(ns('f1'), 1, array('presupplied'), 2);
    foreach (range(100, 300, 100) as $n)
      $x = $x($n);
    $this->assertEquals($x, array(100, 'presupplied', 200, 300));
  }

  function test13() {
    $this->setExpectedException(
      'Exception',
      'Too many arguments accumulated: 4 expected, 6 provided'
    );
    $x = fp\safe_spatter(ns('f1'), 1, array(1), 2);
    $x = $x(111);
    $x = $x(222);
    $x = $x(333, 444, 555);
  }

  function test14() {
    $x = fp\curry('array_map');
    $x = $x('floatval');
    $this->assertEquals($x(range(2, 3)), array(2.0, 3.0));
  }

  function test15() {
    $x = fp\curry('\array_map');
    $x = $x('\floatval');
    $this->assertEquals($x(range(2, 3)), array(2.0, 3.0));
  }

  function test16() {
    $x = fp\safe_spatter('str_replace', array('king'), 3);
    $x = $x('monarch');
    $count = -999;
    $result = $x('The king was beheaded', $count);
    $this->assertEquals($result, 'The monarch was beheaded');
    $this->assertEquals($count, -999);
  }

  function test_autoload() {
    $this->assertFalse(class_exists('c1', FALSE));
    #var_dump(spl_autoload_functions());
    spl_autoload_register('spl_autoload');
    #var_dump(spl_autoload_functions());
    $x = fp\curry(array('c1', 'public_static'));
    $this->assertTrue(class_exists('c1', FALSE));
  }

  function test_function_formats() {
    $c = ns('for_test_function_formats');

    $functions = array(
      array($c, 'public_static'),
      "$c::public_static",
      function ($a, $b) { return array($a, $b); },
      create_function('$a, $b', 'return array($a, $b);'),
    );

    foreach ($functions as $f) {
      $x = fp\curry($f);
      $this->assertEquals($x(1, 2), array(1, 2));
    }

    $functions = array(
      array($c, 'public_instance_method'),
      "$c::public_instance_method",
    );

    foreach ($functions as $f) {
      $x = fp\curry($f);
      $e = error_reporting();
      error_reporting($e & ~E_STRICT);
      $result = $x(1, 2);
      error_reporting($e);
      $this->assertEquals($result, array(1, 2));
    }

    $x = fp\curry(array(new $c, 'public_static'));
    #$e = error_reporting();
    #error_reporting($e & ~E_STRICT);
    $result = $x(1, 2);
    #error_reporting($e);
    $this->assertEquals($result, array(1, 2));
  }

  function test_abstract_method() {
    $this->setExpectedException(
      'Exception',
      "abstract methods are not allowed"
    );
    fp\curry(array(ns('c3'), 'abstract_method'));
  }

  function test_private() {
    $this->setExpectedException(
      'Exception',
      "private methods are not allowed"
    );
    fp\curry(__NAMESPACE__ . '\for_test_private::priv');
  }

  function test_protected() {
    $this->setExpectedException(
      'Exception',
      "protected methods are not allowed"
    );
    fp\curry(__NAMESPACE__ . '\for_test_protected::prot');
  }

  function test_get_reflection() {
    $funcs = array(
      'strpos',
      '\strpos',
      function () {},
      array('DateTime', 'createFromFormat'),
      array('\DateTime', 'createFromFormat'),
      array($this, 'test_get_reflection'),
      array(new \DateTime, 'getOffset'),
    );

    foreach ($funcs as $f) {
      $r = fp\get_reflection($f);
      $this->assertInstanceOf('ReflectionFunctionAbstract', $r);
    }
  }

  function test_return_ref1() {
    $a = 10;
    $b = 20;
    $x = fp\curry(ns('return_ref_to_larger'));
    /*>
      i would prefer to use this, but under PHPUnit it would produce an
      Exception: 

        $result = &$x($a, $b);
    */
    $result = $x($a, $b);
    $this->assertEquals($result, 20);
    $result = -1;
    $this->assertEquals($b, 20);
  }

  function test_return_ref2() {
    $o = new c2;
    $o->n = 777;
    $x = fp\curry(array($o, 'm'));
    $x = $x(1);
    /*>
      i would prefer to use this, but under PHPUnit it would produce an
      Exception: 

        $result = &$x(2);
    */
    $result = $x(2);
    $this->assertEquals($result, 777);
    $result = 888;
    $this->assertEquals($o->n, 777);
  }

  function test_spatter1() {
    $x = fp\spatter(ns('f3'), 2);
    $x = $x(100);
    set_error_handler(
      function ($dummy, $error_message) {
        if(!preg_match('@\bMissing argument\b@', $error_message)) {
          fwrite(STDERR,
            "Fatal error: unexpected error message: $error_message\n");
          exit(1);
        }
      },
      E_WARNING
    );
    $x = $x(200);
    restore_error_handler();
    $this->assertEquals($x, array(array(100, 200), array()));
  }

  function test_spatter2() {
    $x = fp\spatter(ns('f3'), 6);
    foreach (range(101, 106) as $n)
      $x = $x($n);
    $this->assertEquals($x, array(range(101, 103), range(104, 106)));
  }

}

function ns($s) {
  return __NAMESPACE__ . "\\$s";
}

class c {
  static function f1($a, $b, $c, $d = 99) {
    return func_get_args();
  }
}

class c2 {
  public $n;

  function &m($a, $b) {
    return $this->n;
  }
}

abstract class c3 {
  abstract function abstract_method();
}

class for_test_function_formats {
  public static function public_static($a, $b) {
    return array($a, $b);
  }
  public function public_instance_method($a, $b) {
    return array($a, $b);
  }
}

class for_test_private {
  private function priv() {}
}

class for_test_protected {
  protected function prot() {}
}

function f1($a, $b, $c, $d = 99) {
  return func_get_args();
}

function f2(array $a, \DateTime $b, $c, $d = 99) {
  return func_get_args();
}

function f3($a, $b, $c) {
  $a1 = func_get_args();
  $a2 = array_splice($a1, 0, 3);
  return array($a2, $a1);
}

function takes_array_or_NULL(array $a = NULL) {}
function takes_DateTime_or_NULL(\DateTime $d = NULL) {}

function php_uname_wrapper($mode = NULL) {
  return is_null($mode)? php_uname(): php_uname($mode);
}

function &return_ref_to_larger(&$a, &$b) {
  if($a > $b) { return $a; } else { return $b; }
}
